@@ -1,45 +1,44 @@
-##Generalized GWR functions
+##Generalized  GWR functions (basic non-parametric)
 #Author: Binbin Lu
-#Referenced to Chris' R code
-#gwr.fitted <- function(x,b) apply(x*b,1,sum)
+#Referenced to Tomoki Nakaya's c++ code and Chris' R code
 
 ggwr.basic<-function(formula, data, regression.points, bw, family ="poisson", kernel="bisquare",
               adaptive=FALSE, cv=T, tol=1.0e-5, maxiter=20, p=2, theta=0, longlat=F, dMat,dMat1)
 {
-  ##Record the start time
-  timings <- list()
-  timings[["start"]] <- Sys.time()
-  ###################################macth the variables
-  this.call <- match.call()
-  p4s <- as.character(NA)
-  #####Check the given data frame and regression points
-  #####Regression points
-  if (missing(regression.points))
-  {
-  	rp.given <- FALSE
-    regression.points <- data
-    hatmatrix<-T
-  }
-  else 
-  {
-    rp.given <- TRUE
-    hatmatrix<-F
-  }
-  ##Data points{
-  if (is(data, "Spatial"))
-  {
-    p4s <- proj4string(data)
-    dp.locat<-coordinates(data)
-    data <- as(data, "data.frame")
-  }
-  else
-  {
-       stop("Given regression data must be Spatial*DataFrame")
-  }
- 
-  ####################
-  ######Extract the data frame
-  ####Refer to the function lm
+ ##Record the start time
+    timings <- list()
+    timings[["start"]] <- Sys.time()
+    ###################################macth the variables
+    this.call <- match.call()
+    p4s <- as.character(NA)
+    #####Check the given data frame and regression points
+    #####Regression points
+    if (missing(regression.points))
+    {
+        rp.given <- FALSE
+        regression.points <- data
+        hatmatrix<-T
+    }
+    else
+    {
+        rp.given <- TRUE
+        hatmatrix<-F
+    }
+    ##Data points{
+    if (is(data, "Spatial"))
+    {
+        p4s <- proj4string(data)
+        dp.locat<-coordinates(data)
+        data <- as(data, "data.frame")
+    }
+    else
+    {
+        stop("Given regression data must be Spatial*DataFrame")
+    }
+
+    ####################
+    ######Extract the data frame
+    ####Refer to the function lm
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data"), names(mf), 0L)
 
@@ -51,129 +50,127 @@ ggwr.basic<-function(formula, data, regression.points, bw, family ="poisson", ke
     y <- model.extract(mf, "response")
     x <- model.matrix(mt, mf)
     ############################################
-	  var.n<-ncol(x)
+    var.n<-ncol(x)
     if(is(regression.points, "Spatial"))
-    	 rp.locat<-coordinates(regression.points)
+        rp.locat<-coordinates(regression.points)
     else if(is.numeric(regression.points)&&dim(regression.points)[2]==2)
     {
-       rp.locat <- regression.points
+        rp.locat <- regression.points
     }
     else
-       stop("Please use the correct regression points for model calibration!")
-       
-	  rp.n<-nrow(rp.locat)
+        stop("Please use the correct regression points for model calibration!")
+
+    rp.n<-nrow(rp.locat)
     dp.n<-nrow(data)
     betas <-matrix(nrow=rp.n, ncol=var.n)
     betas1<- betas
     if(hatmatrix)
     {
-      betas.SE <-matrix(nrow=rp.n, ncol=var.n)
-      betas.TV <-matrix(nrow=rp.n, ncol=var.n)
-      ##S: hatmatrix
-      S<-matrix(nrow=dp.n,ncol=dp.n)
+        betas.SE <-matrix(nrow=rp.n, ncol=var.n)
+        betas.TV <-matrix(nrow=rp.n, ncol=var.n)
+        ##S: hatmatrix
+        S<-matrix(nrow=dp.n,ncol=dp.n)
     }
     #C.M<-matrix(nrow=dp.n,ncol=dp.n)
     idx1 <- match("(Intercept)", colnames(x))
     if(!is.na(idx1))
-      colnames(x)[idx1]<-"Intercept" 
+        colnames(x)[idx1]<-"Intercept"
     colnames(betas) <- colnames(x)
     #colnames(betas)[1]<-"Intercept"
     ####################################################GWR
-	  #########Distance matrix is given or not
+    #########Distance matrix is given or not
 
     if (missing(dMat))
     {
-      DM.given<-F
-      if(dp.n + rp.n <= 10000)
-      {
-        dMat <- gw.dist(dp.locat=dp.locat, rp.locat=rp.locat, p=p, theta=theta, longlat=longlat)
-        DM.given<-T
-      }      
+        DM.given<-F
+        if(dp.n + rp.n <= 10000)
+        {
+            dMat <- gw.dist(dp.locat=dp.locat, rp.locat=rp.locat, p=p, theta=theta, longlat=longlat)
+            DM.given<-T
+        }
     }
     else
     {
-      DM.given<-T
-      dim.dMat<-dim(dMat)
-      if (dim.dMat[1]!=dp.n||dim.dMat[2]!=rp.n)
-         stop("Dimensions of dMat are not correct")
+        DM.given<-T
+        dim.dMat<-dim(dMat)
+        if (dim.dMat[1]!=dp.n||dim.dMat[2]!=rp.n)
+            stop("Dimensions of dMat are not correct")
     }
     if(missing(dMat1))
     {
-       DM1.given<-F
-       if(hatmatrix&&DM.given)
-       {
-          dMat1 <- dMat
-          DM1.given<-T
-       }
-       else
-       {
-          if(dp.n < 8000)
-          {
-            dMat1 <- gw.dist(dp.locat=dp.locat, rp.locat=dp.locat, p=p, theta=theta, longlat=longlat) 
+        DM1.given<-F
+        if(hatmatrix&&DM.given)
+        {
+            dMat1 <- dMat
             DM1.given<-T
-          } 
-       }
+        }
+        else
+        {
+            if(dp.n < 8000)
+            {
+                dMat1 <- gw.dist(dp.locat=dp.locat, rp.locat=dp.locat, p=p, theta=theta, longlat=longlat)
+                DM1.given<-T
+            }
+        }
     }
     else
     {
-       DM1.given<-T
-       dim.dMat1<-dim(dMat1)
-       if (dim.dMat1[1]!=dp.n||dim.dMat1[2]!=dp.n)
-         stop("Dimensions of dMat are not correct")
-    }  
+        DM1.given<-T
+        dim.dMat1<-dim(dMat1)
+        if (dim.dMat1[1]!=dp.n||dim.dMat1[2]!=dp.n)
+            stop("Dimensions of dMat are not correct")
+    }
     ####Generate the weighting matrix
-     #############Calibration the model
-     W1.mat<-matrix(numeric(dp.n*dp.n),ncol=dp.n)
-     W2.mat<-matrix(numeric(dp.n*rp.n),ncol=rp.n)
+    #############Calibration the model
+    W1.mat<-matrix(numeric(dp.n*dp.n),ncol=dp.n)
+    W2.mat<-matrix(numeric(dp.n*rp.n),ncol=rp.n)
     for (i in 1:dp.n)
     {
-      if (DM1.given)
-         dist.vi<-dMat1[,i]
-      else
-      {
-          dist.vi<-gw.dist(dp.locat=dp.locat, focus=i, p=p, theta=theta, longlat=longlat) 
-      }
-      W.i<-gw.weight(dist.vi,bw,kernel,adaptive)
-      W1.mat[,i]<-W.i
+        if (DM1.given)
+            dist.vi<-dMat1[,i]
+        else
+        {
+            dist.vi<-gw.dist(dp.locat=dp.locat, focus=i, p=p, theta=theta, longlat=longlat)
+        }
+        W.i<-gw.weight(dist.vi,bw,kernel,adaptive)
+        W1.mat[,i]<-W.i
     }
     if (rp.given)
     {
-      for (i in 1:rp.n)
-      {
-        if (DM.given)
-           dist.vi<-dMat[,i]
-        else
+        for (i in 1:rp.n)
         {
-            dist.vi<-gw.dist(dp.locat, rp.locat, focus=i, p, theta, longlat) 
+            if (DM.given)
+                dist.vi<-dMat[,i]
+            else
+            {
+                dist.vi<-gw.dist(dp.locat, rp.locat, focus=i, p, theta, longlat)
+            }
+            W.i<-gw.weight(dist.vi,bw,kernel,adaptive)
+            W2.mat[,i]<-W.i
         }
-        W.i<-gw.weight(dist.vi,bw,kernel,adaptive)
-        W2.mat[,i]<-W.i
-      }
     }
     else
         W2.mat<-W1.mat
+
     ##model calibration
     if(family=="poisson")
-      res1<-gwr.poisson(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol, maxiter)
-    if(family=="binomial")  
-      res1<-gwr.binomial(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol, maxiter)
+        res1<-gwr.poisson(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol, maxiter)
+    if(family=="binomial")
+        res1<-gwr.binomial(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol, maxiter)
     ####################################
     CV <- numeric(dp.n)
     if(hatmatrix && cv)
     {
-       CV <- ggwr.cv.contrib(bw, x, y,family, kernel,adaptive, dp.locat, p, theta, longlat,dMat)
+        CV <- ggwr.cv.contrib(bw, x, y,family, kernel,adaptive, dp.locat, p, theta, longlat,dMat)
     }
-      ####encapsulate the GWR results
-      GW.arguments<-list()
-     GW.arguments<-list(formula=formula,rp.given=rp.given,hatmatrix=hatmatrix,bw=bw, family=family,
-                       kernel=kernel,adaptive=adaptive, p=p, theta=theta, longlat=longlat,DM.given=DM1.given)    
-    
+    ####encapsulate the GWR results
+    GW.arguments<-list()
+    GW.arguments<-list(formula=formula,rp.given=rp.given,hatmatrix=hatmatrix,bw=bw, family=family,
+                       kernel=kernel,adaptive=adaptive, p=p, theta=theta, longlat=longlat,DM.given=DM1.given)
+
     timings[["stop"]] <- Sys.time()
-   ##############
-    if(hatmatrix)
-       res<-list(GW.arguments=GW.arguments,GW.diagnostic=res1$GW.diagnostic,glms=res1$glms,SDF=res1$SDF,CV=CV,timings=timings,this.call=this.call)
-    else
-      res<-list(GW.arguments=GW.arguments,glms=res1$glms,SDF=res1$SDF,CV=CV,timings=timings,this.call=this.call)
+    ##############
+    res<-list(GW.arguments=GW.arguments,GW.diagnostic=res1$GW.diagnostic,glms=res1$glms,SDF=res1$SDF,CV=CV,timings=timings,this.call=this.call)
     class(res) <-"ggwrm"
     invisible(res) 
 }
@@ -183,39 +180,39 @@ gwr.generalised<-function(formula, data, regression.points, bw, family ="poisson
               adaptive=FALSE, cv=T, tol=1.0e-5, maxiter=20, p=2, theta=0, longlat=F, dMat,dMat1)
 {
   ##Record the start time
-  timings <- list()
-  timings[["start"]] <- Sys.time()
-  ###################################macth the variables
-  this.call <- match.call()
-  p4s <- as.character(NA)
-  #####Check the given data frame and regression points
-  #####Regression points
-  if (missing(regression.points))
-  {
-  	rp.given <- FALSE
-    regression.points <- data
-    hatmatrix<-T
-  }
-  else 
-  {
-    rp.given <- TRUE
-    hatmatrix<-F
-  }
-  ##Data points{
-  if (is(data, "Spatial"))
-  {
-    p4s <- proj4string(data)
-    dp.locat<-coordinates(data)
-    data <- as(data, "data.frame")
-  }
-  else
-  {
-       stop("Given regression data must be Spatial*DataFrame")
-  }
- 
-  ####################
-  ######Extract the data frame
-  ####Refer to the function lm
+    timings <- list()
+    timings[["start"]] <- Sys.time()
+    ###################################macth the variables
+    this.call <- match.call()
+    p4s <- as.character(NA)
+    #####Check the given data frame and regression points
+    #####Regression points
+    if (missing(regression.points))
+    {
+        rp.given <- FALSE
+        regression.points <- data
+        hatmatrix<-T
+    }
+    else
+    {
+        rp.given <- TRUE
+        hatmatrix<-F
+    }
+    ##Data points{
+    if (is(data, "Spatial"))
+    {
+        p4s <- proj4string(data)
+        dp.locat<-coordinates(data)
+        data <- as(data, "data.frame")
+    }
+    else
+    {
+        stop("Given regression data must be Spatial*DataFrame")
+    }
+
+    ####################
+    ######Extract the data frame
+    ####Refer to the function lm
     mf <- match.call(expand.dots = FALSE)
     m <- match(c("formula", "data"), names(mf), 0L)
 
@@ -227,135 +224,135 @@ gwr.generalised<-function(formula, data, regression.points, bw, family ="poisson
     y <- model.extract(mf, "response")
     x <- model.matrix(mt, mf)
     ############################################
-	  var.n<-ncol(x)
+    var.n<-ncol(x)
     if(is(regression.points, "Spatial"))
-    	 rp.locat<-coordinates(regression.points)
+        rp.locat<-coordinates(regression.points)
     else if(is.numeric(regression.points)&&dim(regression.points)[2]==2)
     {
-       rp.locat <- regression.points
+        rp.locat <- regression.points
     }
     else
-       stop("Please use the correct regression points for model calibration!")
-       
-	  rp.n<-nrow(rp.locat)
+        stop("Please use the correct regression points for model calibration!")
+
+    rp.n<-nrow(rp.locat)
     dp.n<-nrow(data)
     betas <-matrix(nrow=rp.n, ncol=var.n)
     betas1<- betas
     if(hatmatrix)
     {
-      betas.SE <-matrix(nrow=rp.n, ncol=var.n)
-      betas.TV <-matrix(nrow=rp.n, ncol=var.n)
-      ##S: hatmatrix
-      S<-matrix(nrow=dp.n,ncol=dp.n)
+        betas.SE <-matrix(nrow=rp.n, ncol=var.n)
+        betas.TV <-matrix(nrow=rp.n, ncol=var.n)
+        ##S: hatmatrix
+        S<-matrix(nrow=dp.n,ncol=dp.n)
     }
     #C.M<-matrix(nrow=dp.n,ncol=dp.n)
     idx1 <- match("(Intercept)", colnames(x))
     if(!is.na(idx1))
-      colnames(x)[idx1]<-"Intercept" 
+        colnames(x)[idx1]<-"Intercept"
     colnames(betas) <- colnames(x)
     #colnames(betas)[1]<-"Intercept"
     ####################################################GWR
-	  #########Distance matrix is given or not
+    #########Distance matrix is given or not
 
     if (missing(dMat))
     {
-      DM.given<-F
-      if(dp.n + rp.n <= 10000)
-      {
-        dMat <- gw.dist(dp.locat=dp.locat, rp.locat=rp.locat, p=p, theta=theta, longlat=longlat)
-        DM.given<-T
-      }      
+        DM.given<-F
+        if(dp.n + rp.n <= 10000)
+        {
+            dMat <- gw.dist(dp.locat=dp.locat, rp.locat=rp.locat, p=p, theta=theta, longlat=longlat)
+            DM.given<-T
+        }
     }
     else
     {
-      DM.given<-T
-      dim.dMat<-dim(dMat)
-      if (dim.dMat[1]!=dp.n||dim.dMat[2]!=rp.n)
-         stop("Dimensions of dMat are not correct")
+        DM.given<-T
+        dim.dMat<-dim(dMat)
+        if (dim.dMat[1]!=dp.n||dim.dMat[2]!=rp.n)
+            stop("Dimensions of dMat are not correct")
     }
     if(missing(dMat1))
     {
-       DM1.given<-F
-       if(hatmatrix&&DM.given)
-       {
-          dMat1 <- dMat
-          DM1.given<-T
-       }
-       else
-       {
-          if(dp.n < 8000)
-          {
-            dMat1 <- gw.dist(dp.locat=dp.locat, rp.locat=dp.locat, p=p, theta=theta, longlat=longlat) 
+        DM1.given<-F
+        if(hatmatrix&&DM.given)
+        {
+            dMat1 <- dMat
             DM1.given<-T
-          } 
-       }
+        }
+        else
+        {
+            if(dp.n < 8000)
+            {
+                dMat1 <- gw.dist(dp.locat=dp.locat, rp.locat=dp.locat, p=p, theta=theta, longlat=longlat)
+                DM1.given<-T
+            }
+        }
     }
     else
     {
-       DM1.given<-T
-       dim.dMat1<-dim(dMat1)
-       if (dim.dMat1[1]!=dp.n||dim.dMat1[2]!=dp.n)
-         stop("Dimensions of dMat are not correct")
-    }  
+        DM1.given<-T
+        dim.dMat1<-dim(dMat1)
+        if (dim.dMat1[1]!=dp.n||dim.dMat1[2]!=dp.n)
+            stop("Dimensions of dMat are not correct")
+    }
     ####Generate the weighting matrix
-     #############Calibration the model
-     W1.mat<-matrix(numeric(dp.n*dp.n),ncol=dp.n)
-     W2.mat<-matrix(numeric(dp.n*rp.n),ncol=rp.n)
+    #############Calibration the model
+    W1.mat<-matrix(numeric(dp.n*dp.n),ncol=dp.n)
+    W2.mat<-matrix(numeric(dp.n*rp.n),ncol=rp.n)
     for (i in 1:dp.n)
     {
-      if (DM1.given)
-         dist.vi<-dMat1[,i]
-      else
-      {
-          dist.vi<-gw.dist(dp.locat=dp.locat, focus=i, p=p, theta=theta, longlat=longlat) 
-      }
-      W.i<-gw.weight(dist.vi,bw,kernel,adaptive)
-      W1.mat[,i]<-W.i
+        if (DM1.given)
+            dist.vi<-dMat1[,i]
+        else
+        {
+            dist.vi<-gw.dist(dp.locat=dp.locat, focus=i, p=p, theta=theta, longlat=longlat)
+        }
+        W.i<-gw.weight(dist.vi,bw,kernel,adaptive)
+        W1.mat[,i]<-W.i
     }
     if (rp.given)
     {
-      for (i in 1:rp.n)
-      {
-        if (DM.given)
-           dist.vi<-dMat[,i]
-        else
+        for (i in 1:rp.n)
         {
-            dist.vi<-gw.dist(dp.locat, rp.locat, focus=i, p, theta, longlat) 
+            if (DM.given)
+                dist.vi<-dMat[,i]
+            else
+            {
+                dist.vi<-gw.dist(dp.locat, rp.locat, focus=i, p, theta, longlat)
+            }
+            W.i<-gw.weight(dist.vi,bw,kernel,adaptive)
+            W2.mat[,i]<-W.i
         }
-        W.i<-gw.weight(dist.vi,bw,kernel,adaptive)
-        W2.mat[,i]<-W.i
-      }
     }
     else
         W2.mat<-W1.mat
-    
+
     ##model calibration
     if(family=="poisson")
-      res1<-gwr.poisson(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol, maxiter)
-    if(family=="binomial")  
-      res1<-gwr.binomial(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol, maxiter)
+        res1<-gwr.poisson(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol, maxiter)
+    if(family=="binomial")
+        res1<-gwr.binomial(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol, maxiter)
     ####################################
     CV <- numeric(dp.n)
     if(hatmatrix && cv)
     {
-       CV <- ggwr.cv.contrib(bw, x, y,family, kernel,adaptive, dp.locat, p, theta, longlat,dMat)
+        CV <- ggwr.cv.contrib(bw, x, y,family, kernel,adaptive, dp.locat, p, theta, longlat,dMat)
     }
-      ####encapsulate the GWR results
-      GW.arguments<-list()
-     GW.arguments<-list(formula=formula,rp.given=rp.given,hatmatrix=hatmatrix,bw=bw, family=family,
-                       kernel=kernel,adaptive=adaptive, p=p, theta=theta, longlat=longlat,DM.given=DM1.given)    
-    
+    ####encapsulate the GWR results
+    GW.arguments<-list()
+    GW.arguments<-list(formula=formula,rp.given=rp.given,hatmatrix=hatmatrix,bw=bw, family=family,
+                       kernel=kernel,adaptive=adaptive, p=p, theta=theta, longlat=longlat,DM.given=DM1.given)
+
     timings[["stop"]] <- Sys.time()
-   ##############
+    ##############
     res<-list(GW.arguments=GW.arguments,GW.diagnostic=res1$GW.diagnostic,glms=res1$glms,SDF=res1$SDF,CV=CV,timings=timings,this.call=this.call)
     class(res) <-"ggwrm"
-    invisible(res) 
+    invisible(res)
 }
 
 
 
 ############ Possipon GWGLM
-gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5, maxiter=20)
+gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5, maxiter=500)
 {
     p4s <- as.character(NA)
     if (is(regression.points, "Spatial"))
@@ -364,15 +361,22 @@ gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
     }
     ############################################
     ##Generalized linear regression
-    glms<-glm.fit(x, y, family = poisson())  
+    glms<-glm.fit(x, y, family = poisson()) 
+    null.dev <- glms$null.deviance
+    glm.dev <-glms$deviance
+    glm.pseudo.r2 <- 1- glm.dev/null.dev 
+    glms$pseudo.r2 <- glm.pseudo.r2
+    var.n<-ncol(x)
+    dp.n<-nrow(x)
+    ########change the aic
+    glms$aic <- glm.dev + 2*var.n
+    glms$aicc <- glm.dev + 2*var.n + 2*var.n*(var.n+1)/(dp.n-var.n-1)
     ############################################
-	  var.n<-ncol(x)
     if(is(regression.points, "Spatial"))
     	 rp.locat<-coordinates(regression.points)
     else
        rp.locat <- regression.points
     rp.n<-nrow(rp.locat)
-    dp.n<-nrow(x)
     betas <- matrix(nrow=rp.n, ncol=var.n)
     betas1<- matrix(nrow=dp.n, ncol=var.n)
     betas.SE <-matrix(nrow=dp.n, ncol=var.n)
@@ -384,6 +388,7 @@ gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
    # colnames(betas)[1]<-"Intercept" 
     ####################################
     ##model calibration
+    
     it.count <- 0
     llik <- 0.0
     mu <- y + 0.1
@@ -404,17 +409,32 @@ gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
      llik <- sum(y*nu - mu - log(gamma(y+1)))
      cat(paste("   ",formatC(it.count,digits=4,width=4),"    ",formatC(llik,digits=4,width=7),"\n"))
      if (abs((old.llik - llik)/llik) < tol) break
-     wt2 <- mu
+     wt2 <- as.numeric(mu)
      it.count <- it.count+1
      if (it.count == maxiter) break}
      GW.diagnostic <- NULL
-     if(hatmatrix)
+     gw.dev <- 0
+     for(i in 1:dp.n)
      {
-        for (i in 1:rp.n)
+       if(y[i]!=0)
+           gw.dev <- gw.dev + 2*(y[i]*(log(y[i]/mu[i])-1)+mu[i])
+       else
+           gw.dev <- gw.dev + 2* mu[i]
+     }
+     
+     #gw.dev <- 2*sum(y*log(y/mu)-(y-mu))     
+     #local.dev <- numeric(dp.n)     
+     #local.null.dev <- numeric(dp.n)
+     #local.pseudo.r2 <- numeric(dp.n) 
+     if(hatmatrix)
+     { 
+        for (i in 1:dp.n)
         { 
           W.i<-W2.mat[,i]
           gwsi<-gw_reg(x,y.adj,W.i*wt2,hatmatrix,i)
           betas[i,]<-gwsi[[1]]
+          ##Add the smoother y.adjust, see equation (30) in Nakaya(2005)
+          #S[i,]<-gwsi[[2]]
           S[i,]<-gwsi[[2]]
           Ci<-gwsi[[3]]      
           #betas.SE[i,]<-diag(Ci%*%t(Ci)) 
@@ -422,26 +442,33 @@ gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
           betas.SE[i,] <- diag((Ci*invwt2) %*% t(Ci))# diag(Ci/wt2%*%t(Ci))  #see Nakaya et al. (2005)
         }
         tr.S<-sum(diag(S))
-        tr.StS<-sum(S^2)
+        ####trace(SWS'W^-1) is used here instead of tr.StS
+        #tr.StS<-sum(S^2)
+        tr.StS<- sum(diag(S%*%diag(wt2)%*%t(S)%*% diag(1/wt2)))
         ###edf is different from the definition in Chris' code
-        edf<-dp.n-2*tr.S+tr.StS
+        #edf<-dp.n-2*tr.S+tr.StS
         yhat<-gw.fitted(x, betas)
         residual<-y-exp(yhat)
         ########rss <- sum((y - gwr.fitted(x,b))^2)
-        rss <- sum((y-exp(yhat))^2)
-        sigma.hat <- rss/edf
-        sigma.aic <- rss/dp.n
+        #rss <- sum((y-exp(yhat))^2)
+        #sigma.hat <- rss/edf
+        #sigma.aic <- rss/dp.n
         for(i in 1:dp.n)
         {
-           betas.SE[i,]<-sqrt(sigma.hat*betas.SE[i,])
-           betas.TV[i,]<-betas[i,]/betas.SE[i,] 
+           #betas.SE[i,]<-sqrt(sigma.hat*betas.SE[i,])
+           betas.SE[i,]<-sqrt(betas.SE[i,])
+           betas.TV[i,]<-betas[i,]/betas.SE[i,]  
         }
-        #AICc <- -2*llik + 2*tr.S*dp.n/(dp.n-tr.S-2)
-        AICc <- -2*llik + 2*tr.S + 2*tr.S*(tr.S+1)/(dp.n-tr.S-1)  # This is generic form of AICc (TN)
-        yss.g <- sum((y - mean(y))^2)
-        gw.R2<-1-rss/yss.g; ##R Square valeu
-        gwR2.adj<-1-(1-gw.R2)*(dp.n-1)/(edf-1) #Adjusted R squared value
-        GW.diagnostic<-list(rss=rss,AICc=AICc,edf=edf,gw.R2=gw.R2,gwR2.adj=gwR2.adj)        
+        #AICc <- -2*llik + 2*tr.S*dp.n/(dp.n-tr.S-2) 
+        #AICc <- -2*llik + 2*tr.S + 2*tr.S*(tr.S+1)/(dp.n-tr.S-1)  # This is generic form of AICc (TN)
+        AIC <- gw.dev + 2*tr.S
+        AICc <- gw.dev + 2*tr.S + 2*tr.S*(tr.S+1)/(dp.n-tr.S-1) 
+        #yss.g <- sum((y - mean(y))^2)
+        #gw.R2<-1-rss/yss.g; ##R Square valeu
+        #gwR2.adj<-1-(1-gw.R2)*(dp.n-1)/(edf-1) #Adjusted R squared valu
+        
+        pseudo.R2 <- 1- gw.dev/null.dev
+        GW.diagnostic<-list(gw.deviance=gw.dev,AICc=AICc,AIC=AIC,pseudo.R2 =pseudo.R2)        
      }
      else
      {
@@ -490,22 +517,29 @@ gwr.poisson<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
 }
 
 ############ Binomial GWGLM
-gwr.binomial<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5, maxiter=20)
+
+gwr.binomial <- function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5, maxiter=20)
 {
     p4s <- as.character(NA)
     if (is(regression.points, "Spatial"))
     {
-      p4s <- proj4string(regression.points)
+        p4s <- proj4string(regression.points)
     }
-    
+
     ############################################
     ##Generalized linear regression
-    glms<-glm.fit(x, y, family = binomial())  
-    ############################################
-	  var.n<-ncol(x)
-	  rp.locat<-coordinates(regression.points)
-	  rp.n<-nrow(rp.locat)
+    glms<-glm.fit(x, y, family = binomial())
+    null.dev <- glms$null.deviance
+    glm.dev <-glms$deviance
+    glm.pseudo.r2 <- 1- glm.dev/null.dev
+    glms$pseudo.r2 <- glm.pseudo.r2
+    var.n<-ncol(x)
     dp.n<-nrow(x)
+    glms$aic <- glm.dev + 2*var.n
+    glms$aicc <- glm.dev + 2*var.n + 2*var.n*(var.n+1)/(dp.n-var.n-1)
+    ############################################
+    rp.locat<-coordinates(regression.points)
+    rp.n<-nrow(rp.locat)
     betas <-matrix(nrow=rp.n, ncol=var.n)
     betas1<- matrix(nrow=dp.n, ncol=var.n)
     betas.SE <-matrix(nrow=rp.n, ncol=var.n)
@@ -514,7 +548,7 @@ gwr.binomial<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
     S<-matrix(nrow=dp.n,ncol=dp.n)
     #C.M<-matrix(nrow=dp.n,ncol=dp.n)
     colnames(betas) <- colnames(x)
-    #colnames(betas)[1]<-"Intercept" 
+    #colnames(betas)[1]<-"Intercept"
     ####################################
     ##model calibration
     n=rep(1,length(y))
@@ -525,107 +559,116 @@ gwr.binomial<-function(y,x,regression.points,W1.mat,W2.mat,hatmatrix,tol=1.0e-5,
     cat(" Iteration    Log-Likelihood\n=========================\n")
     wt2 <- rep(1,dp.n)
     repeat {
-     y.adj <- nu + (y - n*mu)/(n*mu*(1 - mu))
-     for (i in 1:dp.n)
-     {
-        W.i<-W1.mat[,i]
-        gwsi<-gw_reg(x,y.adj,W.i*wt2,hatmatrix=F,i)
-        betas1[i,]<-gwsi[[1]]
-     }
-     nu <- gw.fitted(x,betas1)
-     mu <- exp(nu)/(1 + exp(nu))  
-     old.llik <- llik
-     llik <- sum(lchoose(n,y) + (n-y)*log(1 - mu/dp.n) + y*log(mu/n))
-     if(is.na(llik)) llik <-old.llik
-     cat(paste("   ",formatC(it.count,digits=4,width=4),"    ",formatC(llik,digits=4,width=7),"\n"))
-     if (abs((old.llik - llik)/llik) < tol) break
-     wt2 <- n*mu*(1-mu) 
-     it.count <- it.count+1
-     if (it.count == maxiter) break}
-     
-     if(hatmatrix)
-     {
+        y.adj <- nu + (y - n*mu)/(n*mu*(1 - mu))
+        for (i in 1:dp.n)
+        {
+            W.i<-W1.mat[,i]
+            gwsi<-gw_reg(x,y.adj,W.i*wt2,hatmatrix=F,i)
+            betas1[i,]<-gwsi[[1]]
+        }
+        nu <- gw.fitted(x,betas1)
+        mu <- exp(nu)/(1 + exp(nu))
+        old.llik <- llik
+        llik <- sum(lchoose(n,y) + (n-y)*log(1 - mu/n) + y*log(mu/n))
+        if(is.na(llik)) llik <-old.llik
+        cat(paste("   ",formatC(it.count,digits=4,width=4),"    ",formatC(llik,digits=4,width=7),"\n"))
+        if (abs((old.llik - llik)/llik) < tol) break
+        wt2 <- n*mu*(1-mu)
+        #print(length(wt2))
+        it.count <- it.count+1
+        if (it.count == maxiter) break}
+
+    if(hatmatrix)
+    {
         for (i in 1:rp.n)
-        { 
-          W.i<-W1.mat[,i]
-          gwsi<-gw_reg(x,y.adj,W.i*wt2,hatmatrix,i)
-          betas[i,]<-gwsi[[1]] ######See function by IG
-          S[i,]<-gwsi[[2]]
-          Ci<-gwsi[[3]]
-          #betas.SE[i,]<-diag(Ci%*%t(Ci))
-          invwt2 <- 1.0 /as.numeric(wt2)
-          betas.SE[i,] <- diag((Ci*invwt2) %*% t(Ci))   #see Nakaya et al. (2005)
+        {
+            W.i<-W1.mat[,i]
+            gwsi<-gw_reg(x,y.adj,W.i*wt2,hatmatrix,i)
+            betas[i,]<-gwsi[[1]] ######See function by IG
+            #S[i,]<-gwsi[[2]]
+            S[i,]<-gwsi[[2]][1,]
+            Ci<-gwsi[[3]]
+            #betas.SE[i,]<-diag(Ci%*%t(Ci))
+            invwt2 <- 1.0 /as.numeric(wt2)
+            betas.SE[i,] <- diag((Ci*invwt2) %*% t(Ci))   #see Nakaya et al. (2005)
         }
         tr.S<-sum(diag(S))
-        tr.StS<-sum(S^2)
+        #tr.StS<-sum(S^2)
+
+        #tr.StS<- sum(diag(S%*%diag(wt2)%*%t(S)%*% diag(1/wt2)))
         ###edf is different from the definition in Chris' code
-        edf<-dp.n-2*tr.S+tr.StS
+        #edf<-dp.n-2*tr.S+tr.StS
         yhat<-gw.fitted(x, betas)
         residual<-y-exp(yhat)/(1+exp(yhat))
         ########rss <- sum((y - gwr.fitted(x,b))^2)
         rss <- sum(residual^2)
-        sigma.hat <- rss/edf
-        sigma.aic <- rss/dp.n   ### can be omitted? (TN)
+        #sigma.hat <- rss/edf
+        #sigma.aic <- rss/dp.n   ### can be omitted? (TN)
+        gw.dev <- sum(log(1/((y-n+exp(yhat)/(1+exp(yhat))))^2))
         for(i in 1:dp.n)
         {
-           betas.SE[i,]<-sqrt(sigma.hat*betas.SE[i,])
-           betas.TV[i,]<-betas[i,]/betas.SE[i,] 
+            #betas.SE[i,]<-sqrt(sigma.hat*betas.SE[i,])
+            betas.SE[i,]<-sqrt(betas.SE[i,])
+            betas.TV[i,]<-betas[i,]/betas.SE[i,]
         }
         #AICc <- -2*llik + 2*tr.S*dp.n/(dp.n-tr.S-2)
-        AICc <- -2*llik + 2*tr.S + 2*tr.S*(tr.S+1)/(dp.n-tr.S-1)
-        yss.g <- sum((y - mean(y))^2)
-        gw.R2<-1-rss/yss.g; ##R Square valeu  ### is R2 needed? (TN)
-        gwR2.adj<-1-(1-gw.R2)*(dp.n-1)/(edf-1) #Adjusted R squared value
-        GW.diagnostic<-list(rss=rss,AICc=AICc,edf=edf,gw.R2=gw.R2,gwR2.adj=gwR2.adj)        
-     }
-     else
-     {
-        for (i in 1:rp.n)
-        { 
-          W.i<-W2.mat[,i]
-          gwsi<-gw_reg(x,y.adj,W.i*wt2,hatmatrix,i)
-          betas[i,]<-gwsi[[1]]
-        }
-     }
-    if (hatmatrix)                                         
-    {
-      gwres.df<-data.frame(betas,y,exp(yhat)/(1+exp(yhat)),residual,betas.SE,betas.TV)
-      colnames(gwres.df)<-c(c(c(colnames(betas),c("y","yhat","residual")),paste(colnames(betas), "SE", sep="_")),paste(colnames(betas), "TV", sep="_"))
+        #AICc <- -2*llik + 2*tr.S + 2*tr.S*(tr.S+1)/(dp.n-tr.S-1)
+        AICc <- gw.dev + 2*tr.S + 2*tr.S*(tr.S+1)/(dp.n-tr.S-1)
+        AIC <- gw.dev + 2*tr.S
+        #yss.g <- sum((y - mean(y))^2)
+        #gw.R2<-1-rss/yss.g; ##R Square valeu  ### is R2 needed? (TN)
+        #gwR2.adj<-1-(1-gw.R2)*(dp.n-1)/(edf-1) #Adjusted R squared value
+        pseudo.R2 <- 1 - gw.dev/null.dev
+        #GW.diagnostic<-list(rss=rss,AICc=AICc,edf=edf,gw.R2=gw.R2,gwR2.adj=gwR2.adj)
+        GW.diagnostic<-list(gw.deviance=gw.dev,AICc=AICc,AIC=AIC,pseudo.R2 =pseudo.R2)
     }
     else
     {
-      gwres.df<-data.frame(betas)
+        for (i in 1:rp.n)
+        {
+            W.i<-W2.mat[,i]
+            gwsi<-gw_reg(x,y.adj,W.i*wt2,hatmatrix,i)
+            betas[i,]<-gwsi[[1]]
+        }
+    }
+    if(hatmatrix)
+    {
+        gwres.df<-data.frame(betas,y,exp(yhat)/(1+exp(yhat)),residual,betas.SE,betas.TV)
+        colnames(gwres.df)<-c(c(c(colnames(betas),c("y","yhat","residual")),paste(colnames(betas), "SE", sep="_")),paste(colnames(betas), "TV", sep="_"))
+    }
+    else
+    {
+        gwres.df<-data.frame(betas)
     }
     rownames(rp.locat)<-rownames(gwres.df)
-    
+
     griddedObj <- F
     if (is(regression.points, "Spatial"))
-    { 
+    {
         if (is(regression.points, "SpatialPolygonsDataFrame"))
         {
-           polygons<-polygons(regression.points)
-           #SpatialPolygons(regression.points)
-           #rownames(gwres.df) <- sapply(slot(polygons, "polygons"),
-                              #  function(i) slot(i, "ID"))
-           SDF <-SpatialPolygonsDataFrame(Sr=polygons, data=gwres.df,match.ID =F)
+            polygons<-polygons(regression.points)
+            #SpatialPolygons(regression.points)
+            #rownames(gwres.df) <- sapply(slot(polygons, "polygons"),
+            #  function(i) slot(i, "ID"))
+            SDF <-SpatialPolygonsDataFrame(Sr=polygons, data=gwres.df,match.ID =F)
         }
         else
         {
-           griddedObj <- gridded(regression.points)
-           SDF <- SpatialPointsDataFrame(coords=rp.locat, data=gwres.df, proj4string=CRS(p4s), match.ID=F)
-           gridded(SDF) <- griddedObj 
+            griddedObj <- gridded(regression.points)
+            SDF <- SpatialPointsDataFrame(coords=rp.locat, data=gwres.df, proj4string=CRS(p4s), match.ID=F)
+            gridded(SDF) <- griddedObj
         }
     }
     else
         SDF <- SpatialPointsDataFrame(coords=rp.locat, data=gwres.df, proj4string=CRS(p4s), match.ID=F)
-   ##############
+    ##############
     if(hatmatrix)
-       res<-list(GW.diagnostic=GW.diagnostic,glms=glms,SDF=SDF)
+        res<-list(GW.diagnostic=GW.diagnostic,glms=glms,SDF=SDF)
     else
-       res <- list(glms=glms,SDF=SDF)
+        res <- list(glms=glms,SDF=SDF)
 }
 
-
 ############################Layout function for outputing the GWR results
 ##Author: BL	
 print.ggwrm<-function(x, ...)
@@ -649,7 +692,9 @@ print.ggwrm<-function(x, ...)
  	cat("\n   ***********************************************************************\n")
 	  cat("   *              Results of Generalized linear Regression               *\n")
 	  cat("   ***********************************************************************\n")
-	print(summary.glm(x$glms))	
+	print(summary.glm(x$glms))
+  cat("\n AICc: ", x$glms$aicc)
+  cat("\n Pseudo R-square value: ", x$glms$pseudo.r2)
 	#########################################################################
 	cat("\n   ***********************************************************************\n")
 	  cat("   *          Results of Geographically Weighted Regression              *\n")
@@ -705,12 +750,12 @@ print.ggwrm<-function(x, ...)
 	if (x$GW.arguments$hatmatrix) 
   {	
 		cat("   Number of data points:", dp.n, "\n")
-		cat("   Effective degrees of freedom (n-2trace(S) + trace(S'S)):", x$GW.diagnostic$edf, "\n")
-		cat("   AICc :",
+		cat("   GW Deviance:", x$GW.diagnostic$gw.deviance, "\n")
+		cat("   AIC :",
+                    x$GW.diagnostic$AIC, "\n")
+    cat("   AICc :",
                     x$GW.diagnostic$AICc, "\n")
-		cat("   Residual sum of squares:", x$GW.diagnostic$rss, "\n")
-    cat("   R-square value: ",x$GW.diagnostic$gw.R2,"\n")
-		cat("   Adjusted R-square value: ",x$GW.diagnostic$gwR2.adj,"\n")	
+    cat("   Pseudo R-square value: ",x$GW.diagnostic$pseudo.R2,"\n")
   }
 	cat("\n   ***********************************************************************\n")
 	cat("   Program stops at:", as.character(x$timings$stop), "\n")
